@@ -22,23 +22,16 @@ import sys
 import threading
 import traceback
 
-from contextlib import contextmanager
-
 from cloudify_rest_client.executions import Execution
-from cloudify_rest_client.exceptions import (
-    InvalidExecutionUpdateStatus,
-    CloudifyClientError
-)
+from cloudify_rest_client.exceptions import InvalidExecutionUpdateStatus
 
 from cloudify import logs
 from cloudify import exceptions
 from cloudify import state
 from cloudify import context
 from cloudify import utils
-from cloudify import amqp_client_utils
 from cloudify import constants
 from cloudify._compat import queue, StringIO
-from cloudify.amqp_client_utils import AMQPWrappedThread
 from cloudify.manager import update_execution_status, get_rest_client
 from cloudify.constants import LOGGING_CONFIG_FILE
 from cloudify.error_handling import serialize_known_exception
@@ -149,62 +142,13 @@ class OperationHandler(TaskHandler):
             kwargs['ctx'] = ctx
 
         with state.current_ctx.push(ctx, kwargs):
-            # should be single `with` and comma-separate ctxmanagers,
-            # but has to be nested for python 2.6 compat
-            with self._amqp_client():
-                with self._update_operation_state():
-                    self._validate_operation_resumable()
-                    result = self._run_operation_func(ctx, kwargs)
+            self._validate_operation_resumable()
+            result = self._run_operation_func(ctx, kwargs)
 
             if ctx.operation._operation_retry:
                 raise ctx.operation._operation_retry
         return result
 
-    @contextmanager
-    def _update_operation_state(self):
-        ctx = self.ctx
-        store = True
-        try:
-            op = ctx.get_operation()
-        except CloudifyClientError as e:
-            if e.status_code == 404:
-                op = None
-                store = False
-            else:
-                raise
-        if op and op.state == constants.TASK_STARTED:
-            # this operation has been started before? that means we're
-            # resuming a re-delivered operation
-            ctx.resume = True
-        if store:
-            ctx.update_operation(constants.TASK_STARTED)
-        try:
-            yield
-        finally:
-            if store:
-                ctx.update_operation(constants.TASK_RESPONSE_SENT)
-
-    @contextmanager
-    def _amqp_client(self):
-        # initialize an amqp client only when needed, ie. if the task is
-        # not local
-        with_amqp = bool(self.ctx.task_target)
-        if with_amqp:
-            try:
-                amqp_client_utils.init_events_publisher()
-            except Exception:
-                _, ex, tb = sys.exc_info()
-                # This one should never (!) raise an exception.
-                amqp_client_utils.close_amqp_client()
-                raise exceptions.RecoverableError(
-                    'Failed initializing AMQP connection',
-                    causes=[utils.exception_to_error_cause(ex, tb)])
-        try:
-            yield
-        finally:
-            if with_amqp:
-                amqp_client_utils.close_amqp_client()
-
     def _run_operation_func(self, ctx, kwargs):
         try:
             return self.func(*self.args, **kwargs)
@@ -264,7 +208,6 @@ class WorkflowHandler(TaskHandler):
             self.ctx.resume = True
 
         try:
-            amqp_client_utils.init_events_publisher()
             try:
                 self._workflow_started()
             except InvalidExecutionUpdateStatus:
@@ -272,9 +215,10 @@ class WorkflowHandler(TaskHandler):
                 return api.EXECUTION_CANCELLED_RESULT
 
             result_queue = queue.Queue()
-            t = AMQPWrappedThread(target=self._remote_workflow_child_thread,
-                                  args=(result_queue,),
-                                  name='Workflow-Child')
+            t = threading.Thread(target=self._remote_workflow_child_thread,
+                                 args=(result_queue,),
+                                 name='Workflow-Child')
+            t.daemon = True
             t.start()
 
             # while the child thread is executing the workflow, the parent
@@ -336,13 +280,11 @@ class WorkflowHandler(TaskHandler):
                 self._workflow_succeeded()
             return result
         except exceptions.WorkflowFailed as e:
-            self._workflow_failed(e)
+            self._workflow_failed(e, traceback.format_exc())
             raise
         except BaseException as e:
             self._workflow_failed(e, traceback.format_exc())
             raise
-        finally:
-            amqp_client_utils.close_amqp_client()
 
     def _remote_workflow_child_thread(self, queue):
         # the actual execution of the workflow will run in another thread.
@@ -388,19 +330,17 @@ class WorkflowHandler(TaskHandler):
             event_type='workflow_started',
             message="{0} '{1}' workflow execution{2}".format(
                 start_resume, self.ctx.workflow_id, dry_run),
-            additional_context=self._get_hook_params()
         )
 
     def _workflow_succeeded(self):
         self.ctx.cleanup(finished=True)
-        self._update_execution_status(Execution.TERMINATED)
         dry_run = ' (dry run)' if self.ctx.dry_run else ''
         self.ctx.internal.send_workflow_event(
             event_type='workflow_succeeded',
             message="'{0}' workflow execution succeeded{1}".format(
                 self.ctx.workflow_id, dry_run),
-            additional_context=self._get_hook_params()
         )
+        self._update_execution_status(Execution.TERMINATED)
 
     def _workflow_failed(self, exception, error_traceback=None):
         self.ctx.cleanup(finished=True)
@@ -410,7 +350,6 @@ class WorkflowHandler(TaskHandler):
                 message="'{0}' workflow execution failed: {1}".format(
                     self.ctx.workflow_id, exception),
                 args={'error': error_traceback},
-                additional_context=self._get_hook_params()
             )
             self._update_execution_status(Execution.FAILED, error_traceback)
         except Exception:
@@ -421,25 +360,12 @@ class WorkflowHandler(TaskHandler):
 
     def _workflow_cancelled(self):
         self.ctx.cleanup(finished=False)
-        self._update_execution_status(Execution.CANCELLED)
         self.ctx.internal.send_workflow_event(
             event_type='workflow_cancelled',
             message="'{0}' workflow execution cancelled".format(
                 self.ctx.workflow_id),
-            additional_context=self._get_hook_params()
         )
-
-    def _get_hook_params(self):
-        is_system_workflow = self.cloudify_context.get('is_system_workflow')
-        hook_params = {
-            'message_type': 'hook',
-            'is_system_workflow': is_system_workflow,
-            'rest_token': self.cloudify_context.get('rest_token')
-        }
-
-        if not is_system_workflow:
-            hook_params['execution_parameters'] = self.execution_parameters
-        return hook_params
+        self._update_execution_status(Execution.CANCELLED)
 
     def _update_execution_status(self, status, error=None):
         if self.ctx.local or not self.update_execution_status:
