@@ -19,6 +19,7 @@ import random
 import requests
 
 from cloudify._compat import urlquote
+from cloudify.utils import ipv6_url_compat
 
 
 RABBITMQ_MANAGEMENT_PORT = 15671
@@ -30,6 +31,7 @@ class RabbitMQClient(object):
                  port=RABBITMQ_MANAGEMENT_PORT, scheme='https',
                  logger=None, **request_kwargs):
         self._hosts = list(hosts) if isinstance(hosts, list) else [hosts]
+        self._hosts = [ipv6_url_compat(h) for h in self._hosts]
         random.shuffle(self._hosts)
         self._target_host = self._hosts.pop()
         self._port = port
