@@ -414,12 +414,10 @@ class LocalWorkflowTest(BaseWorkflowTest):
     def test_workflow_and_operation_logging_and_events(self):
 
         def assert_task_events(indexes, events):
-            self.assertEqual('sending_task',
-                             events[indexes[0]]['event_type'])
             self.assertEqual('task_started',
-                             events[indexes[1]]['event_type'])
+                             events[indexes[0]]['event_type'])
             self.assertEqual('task_succeeded',
-                             events[indexes[2]]['event_type'])
+                             events[indexes[1]]['event_type'])
 
         def the_workflow(ctx, **_):
             def local_task():
@@ -439,8 +437,7 @@ class LocalWorkflowTest(BaseWorkflowTest):
         with self._mock_stdout_event_and_log() as (events, logs):
             self._execute_workflow(the_workflow, operation_methods=[
                 the_operation])
-
-            self.assertEqual(11, len(events))
+            self.assertEqual(9, len(events))
             self.assertEqual(3, len(logs))
             self.assertEqual('workflow_started',
                              events[0]['event_type'])
@@ -448,12 +445,12 @@ class LocalWorkflowTest(BaseWorkflowTest):
                              events[1]['message']['text'])
             self.assertEqual('node_instance_event',
                              events[2]['message']['text'])
-            assert_task_events([3, 4, 6], events)
+            assert_task_events([3, 5], events)
             self.assertEqual('op_event',
-                             events[5]['message']['text'])
-            assert_task_events([7, 8, 9], events)
+                             events[4]['message']['text'])
+            assert_task_events([6, 7], events)
             self.assertEqual('workflow_succeeded',
-                             events[10]['event_type'])
+                             events[8]['event_type'])
             self.assertEqual('workflow_logging',
                              logs[0]['message']['text'])
             self.assertEqual('node_instance_logging',
@@ -469,7 +466,7 @@ class LocalWorkflowTest(BaseWorkflowTest):
             ctx.local_task(task)
         with self._mock_stdout_event_and_log() as (events, _):
             self._execute_workflow(flow1, use_existing_env=False)
-            self.assertEqual(5, len(events))
+            self.assertEqual(4, len(events))
 
         def flow2(ctx, **_):
             def task():
@@ -499,7 +496,7 @@ class LocalWorkflowTest(BaseWorkflowTest):
 
         with self._mock_stdout_event_and_log() as (events, _):
             self._execute_workflow(flow4, use_existing_env=False)
-            self.assertEqual(5, len(events))
+            self.assertEqual(4, len(events))
 
         def flow5(ctx, **_):
             def task():
@@ -884,14 +881,6 @@ class LocalWorkflowTest(BaseWorkflowTest):
         self.assertRaises(KeyError,
                           storage.get_node, 'node_that_does_not_exist')
 
-    def test_execute_non_existent_operation(self):
-        def flow(ctx, **_):
-            instance = _instance(ctx, 'node')
-            instance.execute_operation('non_existent')
-        with testtools.testcase.ExpectedException(RuntimeError,
-                                                  ".*does not exist.*"):
-            self._execute_workflow(flow)
-
     def test_operation_retry_configuration(self):
         self._test_retry_configuration_impl(
             global_retries=100,
