@@ -43,6 +43,18 @@ from cloudify.utils import get_func
 from cloudify.constants import TASK_RESPONSE_SENT  # noqa
 from cloudify.utils import INSPECT_TIMEOUT  # noqa
 
+try:
+    from functools import lru_cache
+except ImportError:
+    # py2.7 doesn't have lru_cache, but this is only used mgmtworker-side,
+    # on py3 only. Still, this module needs to be importable. Make a noop
+    # function that still has the same interface.
+    def lru_cache():
+        def _inner(f):
+            return f
+        return _inner
+
+
 INFINITE_TOTAL_RETRIES = -1
 DEFAULT_TOTAL_RETRIES = INFINITE_TOTAL_RETRIES
 DEFAULT_RETRY_INTERVAL = 30
@@ -50,6 +62,7 @@ DEFAULT_SUBGRAPH_TOTAL_RETRIES = 0
 
 DEFAULT_SEND_TASK_EVENTS = True
 DISPATCH_TASK = 'cloudify.dispatch.dispatch'
+_NOT_SET = object()
 
 
 def with_execute_after(f):
@@ -266,7 +279,7 @@ class WorkflowTask(object):
         """
         return self._state
 
-    def set_state(self, state):
+    def set_state(self, state, result=_NOT_SET, exception=None):
         """
         Set the task state
 
@@ -277,16 +290,33 @@ class WorkflowTask(object):
                          TASK_RESCHEDULED, TASK_SUCCEEDED, TASK_FAILED]:
             raise RuntimeError('Illegal state set on task: {0} '
                                '[task={1}]'.format(state, str(self)))
-        if self.stored:
-            self._update_stored_state(state)
         if self._state in TERMINATED_STATES:
             return
         self._state = state
+        if self.stored:
+            self._update_stored_state(
+                state, result=result, exception=exception)
+        if not self.stored:
+            event = {}
+            if result is not _NOT_SET:
+                event['result'] = result
+            elif exception:
+                event['exception'] = exception
+            try:
+                self.workflow_context.internal.send_task_event(
+                    state, self, event)
+            except RuntimeError:
+                pass
         if state in TERMINATED_STATES:
             self.is_terminated = True
 
-    def _update_stored_state(self, state):
-        self.workflow_context.update_operation(self.id, state=state)
+    def _update_stored_state(self, state, result=_NOT_SET, exception=None):
+        kwargs = {'state': state}
+        if result is not _NOT_SET:
+            kwargs['result'] = result
+        if exception is not None:
+            kwargs['exception'] = exception
+        self.workflow_context.update_operation(self.id, **kwargs)
 
     def handle_task_terminated(self):
         if self.get_state() in (TASK_FAILED, TASK_RESCHEDULED):
@@ -340,6 +370,7 @@ class WorkflowTask(object):
             # operation explicitly requested a retry, so we ignore
             # the handler set on the task.
             handler_result = HandlerResult.retry()
+            handler_result.retry_after = result.retry_after
         elif self.on_failure:
             handler_result = self.on_failure(self)
         else:
@@ -350,16 +381,6 @@ class WorkflowTask(object):
             elif isinstance(result, exceptions.RecoverableError):
                 handler_result.retry_after = result.retry_after
 
-        if not self.is_subgraph:
-            causes = []
-            if isinstance(result, (exceptions.RecoverableError,
-                                   exceptions.NonRecoverableError)):
-                causes = result.causes or []
-            self.workflow_context.internal.send_task_event(
-                state=self.get_state(),
-                task=self,
-                event={'exception': result, 'causes': causes})
-
         return handler_result
 
     def __str__(self):
@@ -450,20 +471,13 @@ class RemoteWorkflowTask(WorkflowTask):
                 '__cloudify_context'].pop(skipped_field, None)
         return task
 
-    def _update_stored_state(self, state):
-        # no need to store SENDING - all work after SENDING but before SENT
-        # can safely be rerun
-        if state == TASK_SENDING:
-            return
-        return super(RemoteWorkflowTask, self)._update_stored_state(state)
-
     @with_execute_after
     def apply_async(self):
         """Send the task to an agent.
 
         :return: a WorkflowTaskResult instance wrapping the async result
         """
-        should_send = self._state == TASK_PENDING
+        should_send = self._state in (TASK_PENDING, TASK_SENDING)
         if self._state == TASK_PENDING:
             self.set_state(TASK_SENDING)
         try:
@@ -471,8 +485,6 @@ class RemoteWorkflowTask(WorkflowTask):
             self.workflow_context.internal.handler.wait_for_result(
                 self, self._task_target)
             if should_send:
-                self.workflow_context.internal.send_task_event(
-                    TASK_SENDING, self)
                 self.set_state(TASK_SENT)
                 self.workflow_context.internal.handler.send_task(
                     self, self._task_target, self._task_queue)
@@ -538,6 +550,7 @@ class RemoteWorkflowTask(WorkflowTask):
         if rest_host:
             self.kwargs['__cloudify_context']['rest_host'] = rest_host
 
+    @lru_cache()
     def _get_agent_settings(self, node_instance_id, deployment_id,
                             tenant=None):
         """Get the cloudify_agent dict and the tenant dict of the agent.
@@ -664,34 +677,26 @@ class LocalWorkflowTask(WorkflowTask):
 
     def dump(self):
         serialized = super(LocalWorkflowTask, self).dump()
-        if hasattr(self.local_task, 'dump'):
-            serialized_local_task = self.local_task.dump()
-        else:
-            serialized_local_task = None
-        serialized['parameters']['task_kwargs'] = {
-            'name': self._name,
-            'local_task': serialized_local_task
-        }
+        serialized['parameters']['task_kwargs'] = {'name': self._name}
         return serialized
 
     @classmethod
     def restore(cls, ctx, graph, task_descr):
-        local_task_descr = task_descr.parameters['task_kwargs'].get(
-            'local_task')
-        if local_task_descr:
-            local_task = _LocalTask.restore(local_task_descr)
-        else:
-            # task wasn't stored. Noqa because we do want to assign a lambda
-            # here, that is a noop.
-            local_task = lambda *a, **kw: None  # NOQA
+        # task wasn't stored. Noqa because we do want to assign a lambda
+        # here, that is a noop.
+        local_task = lambda *a, **kw: None  # NOQA
         task_descr.parameters['task_kwargs']['local_task'] = local_task
         return super(LocalWorkflowTask, cls).restore(ctx, graph, task_descr)
 
-    def _update_stored_state(self, state):
-        # no need to store SENT - work up to it can safely be redone
-        if state == TASK_SENT:
+    def _update_stored_state(self, state, **kwargs):
+        # no need to store pre-run events, work up to it can safely
+        # be redone; but do save when running cfy local, because then
+        # regular plugin events are this class, and we want to display it
+        if state in (TASK_SENT, TASK_STARTED) \
+                and not self.workflow_context.local:
             return
-        return super(LocalWorkflowTask, self)._update_stored_state(state)
+        return super(LocalWorkflowTask, self)._update_stored_state(
+            state, **kwargs)
 
     @with_execute_after
     def apply_async(self):
@@ -701,20 +706,16 @@ class LocalWorkflowTask(WorkflowTask):
         """
         def local_task_wrapper():
             try:
-                self.workflow_context.internal.send_task_event(TASK_STARTED,
-                                                               self)
+                self.set_state(TASK_STARTED)
                 result = self.local_task(**self.kwargs)
-                self.workflow_context.internal.send_task_event(
-                    TASK_SUCCEEDED, self, event={'result': str(result)})
+                self.set_state(TASK_SUCCEEDED, result=result)
                 self.async_result.result = result
-                self.set_state(TASK_SUCCEEDED)
             except BaseException as e:
                 new_task_state = TASK_RESCHEDULED if isinstance(
                     e, exceptions.OperationRetry) else TASK_FAILED
-                self.set_state(new_task_state)
+                self.set_state(new_task_state, exception=e)
                 self.async_result.result = e
 
-        self.workflow_context.internal.send_task_event(TASK_SENDING, self)
         self.set_state(TASK_SENT)
         self.workflow_context.internal.add_local_task(local_task_wrapper)
         return self.async_result
@@ -750,7 +751,7 @@ class NOPLocalWorkflowTask(WorkflowTask):
         """The task name"""
         return 'NOP'
 
-    def _update_stored_state(self, state):
+    def _update_stored_state(self, state, **kwargs):
         # the task is always stored as pending - nothing to update
         pass
 
@@ -778,13 +779,6 @@ class NOPLocalWorkflowTask(WorkflowTask):
 
 class DryRunLocalWorkflowTask(LocalWorkflowTask):
     def apply_async(self):
-        self.workflow_context.internal.send_task_event(TASK_SENDING, self)
-        self.workflow_context.internal.send_task_event(TASK_STARTED, self)
-        self.workflow_context.internal.send_task_event(
-            TASK_SUCCEEDED,
-            self,
-            event={'result': 'dry run'}
-        )
         self.set_state(TASK_SUCCEEDED)
         self.async_result.result = None
         return self.async_result
@@ -892,56 +886,23 @@ class HandlerResult(object):
         return HandlerResult(cls.HANDLER_IGNORE)
 
 
-# Local tasks implementation
-# A local task is a callable that will be passed to a LocalWorkflowTask,
-# and then executed in a separate thread (see LocalTasksProcessing). Those
-# tasks can implement .restore (classmethod) and .dump methods in order to
-# be resumable.
-# The user-facing interface for those tasks are node_instance.set_state,
-# .send_event, etc.
-
-class _LocalTask(object):
-    """Base class for local tasks, containing utilities."""
-
-    # all local task disable sending task events
-    workflow_task_config = {'send_task_events': False}
+class _BuiltinTaskBase(WorkflowTask):
+    def __init__(self, *args, **kwargs):
+        kwargs.update(send_task_events=False, total_retries=0)
+        super(_BuiltinTaskBase, self).__init__(*args, **kwargs)
 
     @property
-    def __name__(self):
-        # utility, also making subclasses be similar to plain functions
+    def name(self):
         return self.__class__.__name__
 
-    # avoid calling .__subclasses__() many times
-    _subclass_cache = None
+    def _duplicate(self):
+        kwargs = {'workflow_context': self.workflow_context}
+        kwargs.update(self.kwargs)
+        return self.__class__(**kwargs)
 
-    @classmethod
-    def restore(cls, task_descr):
-        """Rehydrate a _LocalTask instance from a dict description.
-
-        The dict will contain a 'task' key and possibly a 'kwargs' key.
-        Choose the appropriate subclass and return an instance of it.
-        """
-        if cls._subclass_cache is None:
-            cls._subclass_cache = dict(
-                (subcls.__name__, subcls) for subcls in cls.__subclasses__()
-            )
-        task_class = cls._subclass_cache[task_descr['task']]
-        kwargs = task_descr.get('kwargs') or {}
-        return task_class(**kwargs)
-
-    # split local/remote on this level. This allows us to reuse implementation,
-    # avoiding the need for separate local/remote subclasses.
-    def __call__(self):
-        if workflow_ctx.local:
-            return self.local()
-        else:
-            return self.remote()
-
-    def local(self):
-        raise NotImplementedError('Implemented by subclasses')
-
-    def remote(self):
-        raise NotImplementedError('Implemented by subclasses')
+    @property
+    def cloudify_context(self):
+        return {}
 
     @property
     def storage(self):
@@ -951,147 +912,145 @@ class _LocalTask(object):
         """
         return workflow_ctx.internal.handler.storage
 
+    @property
+    def rest_client(self):
+        """Shorthand for accessing the rest-client.
 
-class _SetNodeInstanceStateTask(_LocalTask):
-    """A local task that sets a node instance state."""
-
-    def __init__(self, node_instance_id, state):
-        self._node_instance_id = node_instance_id
-        self._state = state
-
-    def __repr__(self):
-        return '<SetNodeInstanceState {0}: {1}>'.format(
-            self._node_instance_id, self._state)
+        Only available in remote workflows.
+        """
+        return workflow_ctx.internal.handler.rest_client
 
     def dump(self):
-        return {
-            'task': self.__name__,
-            'kwargs': {
-                'node_instance_id': self._node_instance_id,
-                'state': self._state
-            }
-        }
+        serialized = super(_BuiltinTaskBase, self).dump()
+        serialized['parameters']['task_kwargs'] = self.kwargs
+        return serialized
+
+    def apply_async(self):
+        self.workflow_context.internal.add_local_task(self._run_task)
+        return self.async_result
+
+    def _run_task(self):
+        func = self.local if self.workflow_context.local else self.remote
+        try:
+            result = func()
+            self.set_state(TASK_SUCCEEDED, result=result)
+            self.async_result.result = result
+        except BaseException as e:
+            self.workflow_context.logger.exception('error running %s', self)
+            self.set_state(TASK_FAILED, exception=e)
+            self.async_result.result = e
 
     def remote(self):
-        get_rest_client().node_instances.update(
-            self._node_instance_id,
+        pass
+
+    def local(self):
+        pass
+
+
+class SetNodeInstanceStateTask(_BuiltinTaskBase):
+    def __init__(self, node_instance_id, state, *args, **kwargs):
+        self.kwargs = {'node_instance_id': node_instance_id, 'state': state}
+        super(SetNodeInstanceStateTask, self).__init__(*args, **kwargs)
+        self.info = state
+
+    def remote(self):
+        if self.stored:
+            # no need to do anything - the server will update the state
+            # automatically once we set this task state to SUCCEEDED
+            return
+        self.rest_client.node_instances.update(
+            self.kwargs['node_instance_id'],
+            state=self.kwargs['state'],
             force=True,
-            state=self._state
         )
 
     def local(self):
         self.storage.update_node_instance(
-            self._node_instance_id,
-            state=self._state,
+            self.kwargs['node_instance_id'],
+            state=self.kwargs['state'],
             version=None)
 
 
-class _GetNodeInstanceStateTask(_LocalTask):
-    """A local task that gets a node instance state."""
-
-    def __init__(self, node_instance_id):
-        self._node_instance_id = node_instance_id
-
-    def dump(self):
-        return {
-            'task': self.__name__,
-            'kwargs': {
-                'node_instance_id': self._node_instance_id
-            }
-        }
+class GetNodeInstanceStateTask(_BuiltinTaskBase):
+    def __init__(self, node_instance_id, *args, **kwargs):
+        self.kwargs = {'node_instance_id': node_instance_id}
+        super(GetNodeInstanceStateTask, self).__init__(*args, **kwargs)
+        self.info = node_instance_id
 
     def remote(self):
-        return get_node_instance(self._node_instance_id).state
+        return get_node_instance(self.kwargs['node_instance_id']).state
 
     def local(self):
-        instance = self.storage.get_node_instance(
-            self._node_instance_id)
-        return instance.state
+        return self.storage.get_node_instance(
+            self.kwargs['node_instance_id']).state
 
 
-class _SendNodeEventTask(_LocalTask):
-    """A local task that sends a node event."""
-    def __init__(self, node_instance_id, event, additional_context):
-        self._node_instance_id = node_instance_id
-        self._event = event
-        self._additional_context = additional_context
-
-    def __repr__(self):
-        return '<SendNodeEvent {0}: "{1}">'.format(
-            self._node_instance_id, self._event)
-
-    def dump(self):
-        return {
-            'task': self.__name__,
-            'kwargs': {
-                'node_instance_id': self._node_instance_id,
-                'event': self._event,
-                'additional_context': self._additional_context
-            }
+class SendNodeEventTask(_BuiltinTaskBase):
+    def __init__(self, node_instance_id, event, additional_context,
+                 *args, **kwargs):
+        self.kwargs = {
+            'node_instance_id': node_instance_id,
+            'event': event,
+            'additional_context': additional_context,
         }
+        super(SendNodeEventTask, self).__init__(*args, **kwargs)
+        self.info = event
 
-    # local/remote only differ by the used output function
     def remote(self):
-        self.send(out_func=logs.amqp_event_out)
+        # only explicitly send the event if the task is not stored - if we do
+        # actually update the operation state, then the server will
+        # automatically add an event
+        if not self.stored:
+            self._send(out_func=logs.manager_event_out)
 
     def local(self):
-        self.send(out_func=logs.stdout_event_out)
+        self._send(out_func=logs.stdout_event_out)
 
-    def send(self, out_func):
+    def _send(self, out_func):
         node_instance = workflow_ctx.get_node_instance(
-            self._node_instance_id)
+            self.kwargs['node_instance_id'])
         logs.send_workflow_node_event(
             ctx=node_instance,
             event_type='workflow_node_event',
-            message=self._event,
-            additional_context=self._additional_context,
+            message=self.kwargs['event'],
+            additional_context=self.kwargs['additional_context'],
             out_func=out_func)
 
 
-class _SendWorkflowEventTask(_LocalTask):
-    """A local task that sends a workflow event."""
-    def __init__(self, event, event_type, args, additional_context=None):
-        self._event = event
-        self._event_type = event_type
-        self._args = args
-        self._additional_context = additional_context
-
-    def dump(self):
-        return {
-            'task': self.__name__,
-            'kwargs': {
-                'event': self._event,
-                'event_type': self._event_type,
-                'args': self._args,
-                'additional_context': self._additional_context
-            }
+class SendWorkflowEventTask(_BuiltinTaskBase):
+    def __init__(self, event, event_type, event_args, additional_context,
+                 *args, **kwargs):
+        self.kwargs = {
+            'event': event,
+            'event_type': event_type,
+            'event_args': event_args,
+            'additional_context': additional_context,
         }
+        super(SendWorkflowEventTask, self).__init__(*args, **kwargs)
+        self.info = event
 
-    def __call__(self):
+    # remote and local are the same
+    def remote(self):
         return workflow_ctx.internal.send_workflow_event(
-            event_type=self._event_type,
-            message=self._event,
-            args=self._args,
-            additional_context=self._additional_context
+            event_type=self.kwargs['event_type'],
+            message=self.kwargs['event'],
+            args=self.kwargs['event_args'],
+            additional_context=self.kwargs['additional_context']
         )
 
+    def local(self):
+        return self.remote()
 
-class _UpdateExecutionStatusTask(_LocalTask):
-    """A local task that sets the execution status."""
-    def __init__(self, status):
-        self._status = status
 
-    def dump(self):
-        return {
-            'task': self.__class__.__name__,
-            'kwargs': {
-                'status': self._status,
-            }
-        }
+class UpdateExecutionStatusTask(_BuiltinTaskBase):
+    def __init__(self, status, *args, **kwargs):
+        self.kwargs = {'status': status}
+        super(UpdateExecutionStatusTask, self).__init__(*args, **kwargs)
+        self.info = status
 
     def remote(self):
         update_execution_status(
-            workflow_ctx.execution_id, self._status)
+            workflow_ctx.execution_id, self.kwargs['status'])
 
     def local(self):
         raise NotImplementedError(
