@@ -40,18 +40,17 @@ from cloudify.workflows.tasks import (TASK_FAILED,
                                       DEFAULT_RETRY_INTERVAL,
                                       DEFAULT_SEND_TASK_EVENTS,
                                       DEFAULT_SUBGRAPH_TOTAL_RETRIES,
-                                      _SetNodeInstanceStateTask,
-                                      _GetNodeInstanceStateTask,
-                                      _SendNodeEventTask,
-                                      _SendWorkflowEventTask,
-                                      _UpdateExecutionStatusTask)
+                                      GetNodeInstanceStateTask,
+                                      SetNodeInstanceStateTask,
+                                      SendNodeEventTask,
+                                      SendWorkflowEventTask,
+                                      UpdateExecutionStatusTask)
 from cloudify.constants import MGMTWORKER_QUEUE
 from cloudify import utils, logs, exceptions
 from cloudify.state import current_workflow_ctx
 from cloudify.workflows import events
 from cloudify.error_handling import deserialize_known_exception
 from cloudify.workflows.tasks_graph import TaskDependencyGraph
-from cloudify.amqp_client_utils import AMQPWrappedThread
 from cloudify.logs import (CloudifyWorkflowLoggingHandler,
                            CloudifyWorkflowNodeLoggingHandler,
                            SystemWideWorkflowLoggingHandler,
@@ -228,48 +227,43 @@ class CloudifyWorkflowNodeInstance(object):
         self._logger = None
 
     def set_state(self, state):
-        """
-        Set the node state
+        """Set the node-instance state
 
-        :param state: The node state
-        :return: the state set
+        :param state: The new node-instance state
+        :return: a state-setting workflow task
         """
         # We don't want to alter the state of the instance during a dry run
         if self.ctx.dry_run:
             return NOPLocalWorkflowTask(self.ctx)
 
-        set_state_task = _SetNodeInstanceStateTask(self.id, state)
-
-        return self.ctx.local_task(
-            local_task=set_state_task,
-            node=self,
-            info=state)
+        return self.ctx._process_task(SetNodeInstanceStateTask(
+            node_instance_id=self.id, state=state,
+            workflow_context=self.ctx,
+        ))
 
     def get_state(self):
-        """
-        Get the node state
+        """Get the node-instance state
 
-        :return: The node state
+        :return: The node-instance state
         """
-        get_state_task = _GetNodeInstanceStateTask(self.id)
-        return self.ctx.local_task(
-            local_task=get_state_task,
-            node=self)
+        return self.ctx._process_task(GetNodeInstanceStateTask(
+            node_instance_id=self.id,
+            workflow_context=self.ctx
+        ))
 
     def send_event(self, event, additional_context=None):
-        """
-        Sends a workflow node event to RabbitMQ
+        """Sends a workflow node event.
 
         :param event: The event
         :param additional_context: additional context to be added to the
                context
         """
-        send_event_task = _SendNodeEventTask(
-            self.id, event, additional_context)
-        return self.ctx.local_task(
-            local_task=send_event_task,
-            node=self,
-            info=event)
+        return self.ctx._process_task(SendNodeEventTask(
+            node_instance_id=self.id,
+            event=event,
+            additional_context=additional_context,
+            workflow_context=self.ctx,
+        ))
 
     def execute_operation(self,
                           operation,
@@ -446,7 +440,6 @@ class _WorkflowContextBase(object):
 
     def __init__(self, ctx, remote_ctx_handler_cls):
         self._context = ctx = ctx or {}
-        self._plugins_cache = {}
         self._local_task_thread_pool_size = ctx.get(
             'local_task_thread_pool_size',
             DEFAULT_LOCAL_TASK_THREAD_POOL_SIZE)
@@ -574,10 +567,8 @@ class _WorkflowContextBase(object):
             target_path=target_path)
 
     def send_event(self, event, event_type='workflow_stage',
-                   args=None,
-                   additional_context=None):
-        """
-        Sends a workflow event to RabbitMQ
+                   args=None, additional_context=None):
+        """Sends a workflow event
 
         :param event: The event
         :param event_type: The event type
@@ -585,25 +576,13 @@ class _WorkflowContextBase(object):
         :param additional_context: additional context to be added to the
                context
         """
-
-        send_event_task = _SendWorkflowEventTask(
-            event, event_type, args, additional_context)
-        return self.local_task(
-            local_task=send_event_task,
-            info=event)
-
-    def _get_plugin(self, plugin):
-        key = (plugin.get('package_name'), plugin.get('package_version'))
-        if key not in self._plugins_cache:
-            client = get_rest_client()
-            filter_plugin = {'package_name': plugin.get('package_name'),
-                             'package_version': plugin.get('package_version')}
-            managed_plugins = client.plugins.list(**filter_plugin)
-            if managed_plugins:
-                plugin['visibility'] = managed_plugins[0]['visibility']
-                plugin['tenant_name'] = managed_plugins[0]['tenant_name']
-            self._plugins_cache[key] = plugin
-        return self._plugins_cache[key]
+        return self._process_task(SendWorkflowEventTask(
+            event=event,
+            event_type=event_type,
+            event_args=args,
+            additional_context=additional_context,
+            workflow_context=self,
+        ))
 
     def _execute_operation(self,
                            operation,
@@ -614,12 +593,8 @@ class _WorkflowContextBase(object):
                            allow_kwargs_override=False,
                            send_task_events=DEFAULT_SEND_TASK_EVENTS):
         kwargs = kwargs or {}
-        op_struct = operations.get(operation)
-        if op_struct is None:
-            raise RuntimeError('{0} operation of node instance {1} does '
-                               'not exist'.format(operation,
-                                                  node_instance.id))
-        if not op_struct['operation']:
+        op_struct = operations.get(operation, {})
+        if not op_struct.get('operation'):
             return NOPLocalWorkflowTask(self)
         plugin_name = op_struct['plugin']
         # could match two plugins with different executors, one is enough
@@ -642,8 +617,8 @@ class _WorkflowContextBase(object):
         else:
             total_retries = operation_total_retries
 
-        if plugin and plugin['package_name'] and not self.local:
-            plugin = self._get_plugin(plugin)
+        if plugin and plugin['package_name']:
+            plugin = self.internal.handler.get_plugin(plugin)
 
         node_context = {
             'node_id': node_instance.id,
@@ -710,16 +685,15 @@ class _WorkflowContextBase(object):
         return result
 
     def update_execution_status(self, new_status):
-        """
-        Updates the execution status to new_status.
+        """Updates the execution status to new_status.
+
         Note that the workflow status gets automatically updated before and
         after its run (whether the run succeeded or failed)
         """
-        update_execution_status_task = _UpdateExecutionStatusTask(new_status)
-
-        return self.local_task(
-            local_task=update_execution_status_task,
-            info=new_status)
+        return self._process_task(UpdateExecutionStatusTask(
+            status=new_status,
+            workflow_context=self,
+        ))
 
     def _build_cloudify_context(self,
                                 task_id,
@@ -896,8 +870,10 @@ class _WorkflowContextBase(object):
     def get_operations(self, graph_id):
         return self.internal.handler.get_operations(graph_id)
 
-    def update_operation(self, operation_id, state):
-        return self.internal.handler.update_operation(operation_id, state)
+    def update_operation(self, operation_id, state,
+                         result=None, exception=None):
+        return self.internal.handler.update_operation(
+            operation_id, state, result, exception)
 
     def get_tasks_graph(self, name):
         return self.internal.handler.get_tasks_graph(self.execution_id, name)
@@ -927,6 +903,7 @@ class _WorkflowContextBase(object):
 class WorkflowNodesAndInstancesContainer(object):
 
     def __init__(self, workflow_context, raw_nodes, raw_node_instances):
+        self.workflow_context = workflow_context
         self._nodes = dict(
             (node.id, CloudifyWorkflowNode(workflow_context, node, self))
             for node in raw_nodes)
@@ -972,14 +949,7 @@ class WorkflowNodesAndInstancesContainer(object):
         return self._node_instances.get(node_instance_id)
 
     def refresh_node_instances(self):
-        if self.local:
-            storage = self.internal.handler.storage
-            raw_node_instances = storage.get_node_instances()
-        else:
-            rest = get_rest_client()
-            raw_node_instances = rest.node_instances.list(
-                deployment_id=self.deployment.id,
-                _get_all_results=True)
+        raw_node_instances = self.internal.handler.get_node_instances()
         self._node_instances = dict(
             (instance.id, CloudifyWorkflowNodeInstance(
                 self, self._nodes[instance.node_id], instance,
@@ -998,6 +968,9 @@ class CloudifyWorkflowContext(
     """
 
     def __init__(self, ctx):
+        self.blueprint = context.BlueprintContext(ctx)
+        self.deployment = WorkflowDeploymentContext(ctx, self)
+
         with current_workflow_ctx.push(self):
             # Not using super() here, because
             # WorkflowNodesAndInstancesContainer's __init__() needs some data
@@ -1006,39 +979,9 @@ class CloudifyWorkflowContext(
             # _WorkflowContextBase, but the way it is now is self-explanatory.
             _WorkflowContextBase.__init__(self, ctx,
                                           RemoteCloudifyWorkflowContextHandler)
-            self.blueprint = context.BlueprintContext(self._context)
-            self.deployment = WorkflowDeploymentContext(self._context, self)
-
-            if self.local:
-                storage = self.internal.handler.storage
-                raw_nodes = storage.get_nodes()
-                raw_node_instances = storage.get_node_instances()
-            elif self.deployment.id:
-                if self.workflow_id in ('create_deployment_environment',
-                                        'delete_deployment_environment'):
-                    # If creating a deployment environment, there are clearly
-                    # no nodes/instances yet.
-                    # If deleting it we don't care about the nodes/instances,
-                    # and trying to retrieve them might cause problems if
-                    # deployment environment creation had a really bad time.
-                    raw_nodes = []
-                    raw_node_instances = []
-                else:
-                    rest = get_rest_client()
-                    dep = self.deployment
-                    raw_nodes = rest.nodes.list(
-                        deployment_id=dep.id,
-                        _get_all_results=True,
-                        evaluate_functions=dep.runtime_only_evaluation)
-                    raw_node_instances = rest.node_instances.list(
-                        deployment_id=dep.id,
-                        _get_all_results=True)
-            else:
-                # no deployment means no nodes
-                # this happens in eg. blueprint-upload
-                raw_nodes = []
-                raw_node_instances = []
 
+            raw_nodes = self.internal.handler.get_nodes()
+            raw_node_instances = self.internal.handler.get_node_instances()
             WorkflowNodesAndInstancesContainer.__init__(self, self, raw_nodes,
                                                         raw_node_instances)
 
@@ -1073,14 +1016,17 @@ class CloudifySystemWideWorkflowContext(_WorkflowContextBase):
 
     @property
     def deployments_contexts(self):
+        if self.local:
+            raise RuntimeError(
+                'deployment_contexts do not exist in local workflows')
         if self._dep_contexts is None:
             self._dep_contexts = {}
 
-            rest = get_rest_client(tenant=self.tenant_name)
-            deployments_list = rest.deployments.list(
-                _include=['id', 'blueprint_id'],
-                _get_all_results=True
-            )
+            deployments_list = \
+                self.internal.handler.rest_client.deployments.list(
+                    _include=['id', 'blueprint_id'],
+                    _get_all_results=True
+                )
             for dep in deployments_list:
                 # Failure to deepcopy will cause snapshot restore context hack
                 # to be reset just before it's needed.
@@ -1189,26 +1135,19 @@ class LocalTasksProcessing(object):
     def __init__(self, workflow_ctx, thread_pool_size=1):
         self._local_tasks_queue = queue.Queue()
         self._local_task_processing_pool = []
+        self.workflow_ctx = workflow_ctx
         self._is_local_context = workflow_ctx.local
         for i in range(thread_pool_size):
             name = 'Task-Processor-{0}'.format(i + 1)
-            if self._is_local_context:
-                thread = threading.Thread(target=self._process_local_task,
-                                          name=name, args=(workflow_ctx, ))
-                thread.daemon = True
-            else:
-                # this is a remote workflow, use an AMQPWrappedThread
-                thread = AMQPWrappedThread(target=self._process_local_task,
-                                           name=name, args=(workflow_ctx, ))
+            thread = threading.Thread(target=self._process_local_task,
+                                      name=name, args=(workflow_ctx, ))
+            thread.daemon = True
             self._local_task_processing_pool.append(thread)
         self.stopped = False
 
     def start(self):
         for thread in self._local_task_processing_pool:
             thread.start()
-        if not self._is_local_context:
-            for thread in self._local_task_processing_pool:
-                thread.started_amqp_client.wait(timeout=30)
 
     def stop(self):
         self.stopped = True
@@ -1228,80 +1167,6 @@ class LocalTasksProcessing(object):
                 except Exception:
                     pass
 
-# Local/Remote Handlers
-
-
-class CloudifyWorkflowContextHandler(object):
-
-    def __init__(self, workflow_ctx):
-        self.workflow_ctx = workflow_ctx
-
-    def cleanup(self, finished):
-        pass
-
-    def get_context_logging_handler(self):
-        raise NotImplementedError('Implemented by subclasses')
-
-    def get_node_logging_handler(self, workflow_node_instance):
-        raise NotImplementedError('Implemented by subclasses')
-
-    @property
-    def bootstrap_context(self):
-        raise NotImplementedError('Implemented by subclasses')
-
-    def get_send_task_event_func(self, task):
-        raise NotImplementedError('Implemented by subclasses')
-
-    @property
-    def operation_cloudify_context(self):
-        raise NotImplementedError('Implemented by subclasses')
-
-    def send_workflow_event(self, event_type, message=None, args=None,
-                            additional_context=None):
-        raise NotImplementedError('Implemented by subclasses')
-
-    def download_deployment_resource(self,
-                                     resource_path,
-                                     target_path=None):
-        raise NotImplementedError('Implemented by subclasses')
-
-    def start_deployment_modification(self, nodes):
-        raise NotImplementedError('Implemented by subclasses')
-
-    def finish_deployment_modification(self, modification):
-        raise NotImplementedError('Implemented by subclasses')
-
-    def rollback_deployment_modification(self, modification):
-        raise NotImplementedError('Implemented by subclasses')
-
-    def list_deployment_modifications(self, status):
-        raise NotImplementedError('Implemented by subclasses')
-
-    def scaling_groups(self):
-        raise NotImplementedError('Implemented by subclasses')
-
-    def get_operations(self, graph_id):
-        raise NotImplementedError('Implemented by subclasses')
-
-    def get_tasks_graph(self, execution_id, name):
-        raise NotImplementedError('Implemented by subclasses')
-
-    def update_operation(self, operation_id, state):
-        raise NotImplementedError('Implemented by subclasses')
-
-    def store_tasks_graph(self, execution_id, name, operations):
-        raise NotImplementedError('Implemented by subclasses')
-
-    def store_operation(self, graph_id, dependencies,
-                        id, name, type, parameters, **kwargs):
-        raise NotImplementedError('Implemented by subclasses')
-
-    def remove_operation(self, operation_id):
-        raise NotImplementedError('Implemented by subclasses')
-
-    def get_execution(self, execution_id):
-        raise NotImplementedError('Implemented by subclasses')
-
 
 class _WorkflowTaskHandler(object):
     def __init__(self, workflow_ctx):
@@ -1405,26 +1270,21 @@ class _WorkflowTaskHandler(object):
                     state = TASK_RESCHEDULED
                 else:
                     state = TASK_FAILED
-                self._set_task_state(task, state)
+                self._set_task_state(task, state, exception=exception)
                 task.async_result.result = exception
             else:
                 state = TASK_SUCCEEDED
-                self._set_task_state(task, state, {
-                    'result': response.get('result')
-                })
-                task.async_result.result = response.get('result')
+                result = response.get('result')
+                self._set_task_state(task, state, result=result)
+                task.async_result.result = result
         except Exception:
             self._logger.error('Error occurred while processing task',
                                exc_info=True)
             raise
 
-    def _set_task_state(self, task, state, event=None):
+    def _set_task_state(self, task, state, **kwargs):
         with current_workflow_ctx.push(task.workflow_context):
-            task.set_state(state)
-            if event is not None:
-                events.send_task_event(
-                    state, task,
-                    events.send_task_event_func_remote, event)
+            task.set_state(state, **kwargs)
 
 
 class _TaskDispatcher(object):
@@ -1434,8 +1294,9 @@ class _TaskDispatcher(object):
         self._clients = {}
 
     def cleanup(self):
-        for _client, handler in self._clients.values():
+        for client, handler in self._clients.values():
             handler.delete_queue()
+            client.close()
 
     def get_client(self, target):
         if target == MGMTWORKER_QUEUE:
@@ -1481,10 +1342,96 @@ class _TaskDispatcher(object):
         handler.wait_for_task(task)
 
 
+# Local/Remote Handlers
+
+class CloudifyWorkflowContextHandler(object):
+
+    def __init__(self, workflow_ctx):
+        self.workflow_ctx = workflow_ctx
+
+    def cleanup(self, finished):
+        pass
+
+    def get_context_logging_handler(self):
+        raise NotImplementedError('Implemented by subclasses')
+
+    def get_node_logging_handler(self, workflow_node_instance):
+        raise NotImplementedError('Implemented by subclasses')
+
+    @property
+    def bootstrap_context(self):
+        raise NotImplementedError('Implemented by subclasses')
+
+    def get_send_task_event_func(self, task):
+        raise NotImplementedError('Implemented by subclasses')
+
+    @property
+    def operation_cloudify_context(self):
+        raise NotImplementedError('Implemented by subclasses')
+
+    def send_workflow_event(self, event_type, message=None, args=None,
+                            additional_context=None):
+        raise NotImplementedError('Implemented by subclasses')
+
+    def download_deployment_resource(self,
+                                     resource_path,
+                                     target_path=None):
+        raise NotImplementedError('Implemented by subclasses')
+
+    def start_deployment_modification(self, nodes):
+        raise NotImplementedError('Implemented by subclasses')
+
+    def finish_deployment_modification(self, modification):
+        raise NotImplementedError('Implemented by subclasses')
+
+    def rollback_deployment_modification(self, modification):
+        raise NotImplementedError('Implemented by subclasses')
+
+    def list_deployment_modifications(self, status):
+        raise NotImplementedError('Implemented by subclasses')
+
+    def scaling_groups(self):
+        raise NotImplementedError('Implemented by subclasses')
+
+    def get_operations(self, graph_id):
+        raise NotImplementedError('Implemented by subclasses')
+
+    def get_tasks_graph(self, execution_id, name):
+        raise NotImplementedError('Implemented by subclasses')
+
+    def update_operation(self, operation_id, state,
+                         result=None, exception=None):
+        raise NotImplementedError('Implemented by subclasses')
+
+    def store_tasks_graph(self, execution_id, name, operations):
+        raise NotImplementedError('Implemented by subclasses')
+
+    def store_operation(self, graph_id, dependencies,
+                        id, name, type, parameters, **kwargs):
+        raise NotImplementedError('Implemented by subclasses')
+
+    def remove_operation(self, operation_id):
+        raise NotImplementedError('Implemented by subclasses')
+
+    def get_execution(self, execution_id):
+        raise NotImplementedError('Implemented by subclasses')
+
+    def get_nodes(self):
+        raise NotImplementedError('Implemented by subclasses')
+
+    def get_node_instances(self):
+        raise NotImplementedError('Implemented by subclasses')
+
+    def get_plugin(self, plugin_spec):
+        raise NotImplementedError('Implemented by subclasses')
+
+
 class RemoteContextHandler(CloudifyWorkflowContextHandler):
     def __init__(self, *args, **kwargs):
         super(RemoteContextHandler, self).__init__(*args, **kwargs)
         self._dispatcher = _TaskDispatcher(self.workflow_ctx)
+        self.rest_client = get_rest_client()
+        self._plugins_cache = {}
 
     def cleanup(self, finished):
         if finished:
@@ -1528,11 +1475,11 @@ class RemoteContextHandler(CloudifyWorkflowContextHandler):
                                  logger=logger)
 
     def get_operations(self, graph_id):
-        client = get_rest_client()
         ops = []
         offset = 0
         while True:
-            operations = client.operations.list(graph_id, _offset=offset)
+            operations = self.rest_client.operations.list(
+                graph_id, _offset=offset)
             ops += operations.items
             if len(ops) < operations.metadata.pagination.total:
                 offset += operations.metadata.pagination.size
@@ -1540,24 +1487,34 @@ class RemoteContextHandler(CloudifyWorkflowContextHandler):
                 break
         return ops
 
-    def update_operation(self, operation_id, state):
-        client = get_rest_client()
-        client.operations.update(operation_id, state=state)
+    def update_operation(self, operation_id, state,
+                         result=None, exception=None):
+        exception_causes = None
+        exception_text = None
+        try:
+            json.dumps(result)
+        except ValueError:
+            # it's not serializable! just store a null (as is back-compatible)
+            result = None
+        if exception is not None:
+            exception_text = str(exception)
+            exception_causes = getattr(exception, 'causes', None)
+        self.rest_client.operations.update(
+            operation_id, state=state, result=result,
+            exception=exception_text, exception_causes=exception_causes)
 
     def get_tasks_graph(self, execution_id, name):
-        client = get_rest_client()
-        graphs = client.tasks_graphs.list(execution_id, name)
+        graphs = self.rest_client.tasks_graphs.list(execution_id, name)
         if graphs:
             return graphs[0]
 
     def store_tasks_graph(self, execution_id, name, operations):
-        client = get_rest_client()
-        return client.tasks_graphs.create(execution_id, name, operations)
+        return self.rest_client.tasks_graphs.create(
+            execution_id, name, operations)
 
     def store_operation(self, graph_id, dependencies,
                         id, name, type, parameters, **kwargs):
-        client = get_rest_client()
-        client.operations.create(
+        self.rest_client.operations.create(
             operation_id=id,
             graph_id=graph_id,
             name=name,
@@ -1566,25 +1523,60 @@ class RemoteContextHandler(CloudifyWorkflowContextHandler):
             parameters=parameters)
 
     def remove_operation(self, operation_id):
-        client = get_rest_client()
-        client.operations.delete(operation_id)
+        self.rest_client.operations.delete(operation_id)
 
     def get_execution(self, execution_id):
-        client = get_rest_client()
-        return client.executions.get(execution_id)
+        return self.rest_client.executions.get(execution_id)
+
+    def get_nodes(self):
+        if self.workflow_ctx.workflow_id in ('create_deployment_environment',
+                                             'delete_deployment_environment'):
+            # If creating a deployment environment, there are clearly
+            # no nodes/instances yet.
+            # If deleting it we don't care about the nodes/instances,
+            # and trying to retrieve them might cause problems if
+            # deployment environment creation had a really bad time.
+            return []
+        dep = self.workflow_ctx.deployment
+        return self.rest_client.nodes.list(
+            deployment_id=dep.id,
+            _get_all_results=True,
+            evaluate_functions=dep.runtime_only_evaluation
+        )
+
+    def get_node_instances(self):
+        if self.workflow_ctx.workflow_id in ('create_deployment_environment',
+                                             'delete_deployment_environment'):
+            return []
+        dep = self.workflow_ctx.deployment
+        return self.rest_client.node_instances.list(
+            deployment_id=dep.id,
+            _get_all_results=True,
+        )
+
+    def get_plugin(self, plugin):
+        key = (plugin.get('package_name'), plugin.get('package_version'))
+        if key not in self._plugins_cache:
+            filter_plugin = {'package_name': plugin.get('package_name'),
+                             'package_version': plugin.get('package_version')}
+            managed_plugins = self.rest_client.plugins.list(**filter_plugin)
+            if managed_plugins:
+                plugin['visibility'] = managed_plugins[0]['visibility']
+                plugin['tenant_name'] = managed_plugins[0]['tenant_name']
+            self._plugins_cache[key] = plugin
+        return self._plugins_cache[key]
 
 
 class RemoteCloudifyWorkflowContextHandler(RemoteContextHandler):
-
     _scaling_groups = None
 
     def get_node_logging_handler(self, workflow_node_instance):
-        return CloudifyWorkflowNodeLoggingHandler(workflow_node_instance,
-                                                  out_func=logs.amqp_log_out)
+        return CloudifyWorkflowNodeLoggingHandler(
+            workflow_node_instance, out_func=logs.manager_log_out)
 
     def get_context_logging_handler(self):
         return CloudifyWorkflowLoggingHandler(self.workflow_ctx,
-                                              out_func=logs.amqp_log_out)
+                                              out_func=logs.manager_log_out)
 
     def download_deployment_resource(self,
                                      resource_path,
@@ -1599,8 +1591,7 @@ class RemoteCloudifyWorkflowContextHandler(RemoteContextHandler):
 
     def start_deployment_modification(self, nodes):
         deployment_id = self.workflow_ctx.deployment.id
-        client = get_rest_client()
-        modification = client.deployment_modifications.start(
+        modification = self.rest_client.deployment_modifications.start(
             deployment_id=deployment_id,
             nodes=nodes,
             context={
@@ -1612,17 +1603,14 @@ class RemoteCloudifyWorkflowContextHandler(RemoteContextHandler):
         return Modification(self.workflow_ctx, modification)
 
     def finish_deployment_modification(self, modification):
-        client = get_rest_client()
-        client.deployment_modifications.finish(modification.id)
+        self.rest_client.deployment_modifications.finish(modification.id)
 
     def rollback_deployment_modification(self, modification):
-        client = get_rest_client()
-        client.deployment_modifications.rollback(modification.id)
+        self.rest_client.deployment_modifications.rollback(modification.id)
 
     def list_deployment_modifications(self, status):
         deployment_id = self.workflow_ctx.deployment.id
-        client = get_rest_client()
-        modifications = client.deployment_modifications.list(
+        modifications = self.rest_client.deployment_modifications.list(
             deployment_id=deployment_id,
             status=status)
         return [Modification(self.workflow_ctx, m) for m in modifications]
@@ -1634,14 +1622,13 @@ class RemoteCloudifyWorkflowContextHandler(RemoteContextHandler):
                             message=message,
                             args=args,
                             additional_context=additional_context,
-                            out_func=logs.amqp_event_out)
+                            out_func=logs.manager_event_out)
 
     @property
     def scaling_groups(self):
         if not self._scaling_groups:
             deployment_id = self.workflow_ctx.deployment.id
-            client = get_rest_client()
-            deployment = client.deployments.get(
+            deployment = self.rest_client.deployments.get(
                 deployment_id, _include=['scaling_groups'])
             self._scaling_groups = deployment['scaling_groups']
         return self._scaling_groups
@@ -1651,7 +1638,7 @@ class SystemWideWfRemoteContextHandler(RemoteContextHandler):
 
     def get_context_logging_handler(self):
         return SystemWideWorkflowLoggingHandler(self.workflow_ctx,
-                                                out_func=logs.amqp_log_out)
+                                                out_func=logs.manager_log_out)
 
     def send_workflow_event(self, event_type, message=None, args=None,
                             additional_context=None):
@@ -1660,7 +1647,7 @@ class SystemWideWfRemoteContextHandler(RemoteContextHandler):
                                message=message,
                                args=args,
                                additional_context=additional_context,
-                               out_func=logs.amqp_event_out)
+                               out_func=logs.manager_event_out)
 
 
 class LocalCloudifyWorkflowContextHandler(CloudifyWorkflowContextHandler):
@@ -1714,7 +1701,8 @@ class LocalCloudifyWorkflowContextHandler(CloudifyWorkflowContextHandler):
     def get_tasks_graph(self, execution_id, name):
         pass
 
-    def update_operation(self, operation_id, state):
+    def update_operation(self, operation_id, state,
+                         result=None, exception=None):
         pass
 
     def store_tasks_graph(self, execution_id, name, operations):
@@ -1730,6 +1718,15 @@ class LocalCloudifyWorkflowContextHandler(CloudifyWorkflowContextHandler):
     def get_execution(self, execution_id):
         return self.storage.get_execution(execution_id)
 
+    def get_nodes(self):
+        return self.storage.get_nodes()
+
+    def get_node_instances(self):
+        return self.storage.get_node_instances()
+
+    def get_plugin(self, plugin):
+        return plugin
+
 
 class Modification(object):
 
